@@ -190,6 +190,8 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 	
 	home = str(Path.home())
 	
+	img_region = [0, 0, img.shape[0], img.shape[1]]
+	
 	#if functions.preproces_face is called directly, then face_detector global variable might not been initialized.
 	if not "face_detector" in globals():
 		initialize_detector(detector_backend = detector_backend)
@@ -206,12 +208,12 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 		if len(faces) > 0:
 			x,y,w,h = faces[0] #focus on the 1st face found in the image
 			detected_face = img[int(y):int(y+h), int(x):int(x+w)]
-			return detected_face
+			return detected_face, [x, y, w, h]
 		
 		else: #if no face detected
 	
 			if enforce_detection != True:			
-				return img
+				return img, img_region
 	
 			else:
 				raise ValueError("Face could not be detected. Please confirm that the picture is a face photo or consider to set enforce_detection param to False.")
@@ -260,13 +262,13 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 			
 			detected_face = base_img[int(top*aspect_ratio_y):int(bottom*aspect_ratio_y), int(left*aspect_ratio_x):int(right*aspect_ratio_x)]
 			
-			return detected_face
+			return detected_face, [int(left*aspect_ratio_x), int(top*aspect_ratio_y), int(right*aspect_ratio_x) - int(left*aspect_ratio_x), int(bottom*aspect_ratio_y) - int(top*aspect_ratio_y)]
 			
 		else: #if no face detected
 	
 			if enforce_detection != True:
 				img = base_img.copy()
-				return img
+				return img, img_region
 	
 			else:
 				raise ValueError("Face could not be detected. Please confirm that the picture is a face photo or consider to set enforce_detection param to False.")
@@ -283,12 +285,12 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 				
 				detected_face = img[top:bottom, left:right]
 				
-				return detected_face
+				return detected_face, [left, top, right - left, bottom - top]
 			
 		else: #if no face detected
 	
 			if enforce_detection != True:			
-				return img
+				return img, img_region
 	
 			else:
 				raise ValueError("Face could not be detected. Please confirm that the picture is a face photo or consider to set enforce_detection param to False.") 
@@ -302,11 +304,11 @@ def detect_face(img, detector_backend = 'opencv', grayscale = False, enforce_det
 			detection = detections[0]
 			x, y, w, h = detection["box"]
 			detected_face = img[int(y):int(y+h), int(x):int(x+w)]
-			return detected_face
+			return detected_face, [x, y, w, h]
 		
 		else: #if no face detected
 			if not enforce_detection:			
-				return img
+				return img, img_region
 	
 			else:
 				raise ValueError("Face could not be detected. Please confirm that the picture is a face photo or consider to set enforce_detection param to False.")
@@ -436,7 +438,7 @@ def align_face(img, detector_backend = 'opencv'):
 				
 		return img #return img anyway
 	
-def preprocess_face(img, target_size=(224, 224), grayscale = False, enforce_detection = True, detector_backend = 'opencv'):
+def preprocess_face(img, target_size=(224, 224), grayscale = False, enforce_detection = True, detector_backend = 'opencv', return_region = False):
 	
 	#img_path = copy.copy(img)
 	
@@ -444,7 +446,7 @@ def preprocess_face(img, target_size=(224, 224), grayscale = False, enforce_dete
 	img = load_image(img)
 	base_img = img.copy()
 	
-	img = detect_face(img = img, detector_backend = detector_backend, grayscale = grayscale, enforce_detection = enforce_detection)
+	img, region = detect_face(img = img, detector_backend = detector_backend, grayscale = grayscale, enforce_detection = enforce_detection)
 	
 	#--------------------------
 	
@@ -468,7 +470,10 @@ def preprocess_face(img, target_size=(224, 224), grayscale = False, enforce_dete
 	img_pixels = np.expand_dims(img_pixels, axis = 0)
 	img_pixels /= 255 #normalize input in [0, 1]
 	
-	return img_pixels
+	if return_region == True:
+		return img_pixels, region
+	else:
+		return img_pixels
 
 def find_input_shape(model):
 	
@@ -481,6 +486,9 @@ def find_input_shape(model):
 		input_shape = input_shape[0][1:3]
 	else:
 		input_shape = input_shape[1:3]
+		
+	if type(input_shape) == list: #issue 197: some people got array here instead of tuple
+		input_shape = tuple(input_shape)
 	
 	return input_shape
 	
